package twentyonecoders.GUI;

import javafx.animation.*;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.shape.QuadCurve;
import javafx.geometry.Pos;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.image.Image;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.util.Duration;

import twentyonecoders.Controller;
import twentyonecoders.GUI.components.CardUI;
import twentyonecoders.GUI.components.CustomDialog;
import twentyonecoders.logic.Card;
import twentyonecoders.logic.Config;
import twentyonecoders.logic.Player;
import twentyonecoders.utils.LogUtil;

/**
 * @author Lisa Mussner
 */
public class LandingPage extends BorderPane {

    private final Label TITLE_LABEL;
    private final TextField USERNAME_TEXTFIELD;
    private final Button START_BUTTON;
    private final CardUI CARD;
    private final CardUI CARD1;
    private final CardUI CARD2;
    private final CardUI CARD3;
    private final CardUI CARD4;
    private final Controller CONTROLLER;
    private final LogUtil LOG = new LogUtil();

    /**
     * Creates and styles the Landingpage.
     * @param controller that controls the flow of the game.
     */
    public LandingPage(Controller controller) {
        this.CONTROLLER = controller;
        this.setHeight(Config.SCREEN_HEIGHT);
        this.setWidth(Config.SCREEN_WIDTH);
        this.setBackground(
                new Background(
                        new BackgroundImage(
                                new Image(String.valueOf(this.getClass().getResource("/Images/menu_bg.png"))),
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                )
        );

        //title
        TITLE_LABEL = new Label("BlackJack");
        TITLE_LABEL.setTextFill(Color.rgb(206, 255, 26));
        TITLE_LABEL.setFont(new Font("Arial Bold", 80));

        //text-field for username
        USERNAME_TEXTFIELD = new TextField();
        USERNAME_TEXTFIELD.setMinHeight(50);
        USERNAME_TEXTFIELD.setMaxWidth(300);
        USERNAME_TEXTFIELD.setAlignment(Pos.CENTER);
        USERNAME_TEXTFIELD.setFont(new Font("Arial Bold", 25));
        USERNAME_TEXTFIELD.setStyle("-fx-text-fill: #CEFF1A; -fx-prompt-text-fill: #CEFF1A;");
        USERNAME_TEXTFIELD.setBackground(
                new Background(
                        new BackgroundImage(
                                new Image(String.valueOf(this.getClass().getResource("/Images/txt_bg.png"))),
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                )
        );

        //start button
        START_BUTTON = new Button();
        START_BUTTON.setMinWidth(220);
        START_BUTTON.setMinHeight(60);
        START_BUTTON.setBackground(
                new Background(
                        new BackgroundImage(
                                new Image(String.valueOf(this.getClass().getResource("/Images/btn_start.png"))),
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                )
        );
        START_BUTTON.setOnMouseClicked(this::start);

        //cardStates for animation
        CARD = new CardUI(new Card(Card.Suit.HEARTS, Card.Rank.ACE));
        CARD1 = new CardUI(new Card(Card.Suit.HEARTS, Card.Rank.NINE));
        CARD2 = new CardUI(new Card(Card.Suit.CLUBS, Card.Rank.ACE));
        CARD3 = new CardUI(new Card(Card.Suit.DIAMONDS, Card.Rank.JACK));
        CARD4 = new CardUI(new Card(Card.Suit.SPADES, Card.Rank.TWO));

        //card deck
        RotateTransition rt = new RotateTransition(Duration.seconds(0), CARD);
        rt.setFromAngle(-35);
        rt.setToAngle(-35);
        rt.setCycleCount(1);

        QuadCurve qc = new QuadCurve();
        qc.setStartX(-50f);
        qc.setStartY(100f);
        qc.setEndX(-50f);
        qc.setEndY(100f);

        PathTransition pt = new PathTransition();
        pt.setNode(CARD);
        pt.setDuration(Duration.seconds(0));
        pt.setPath(qc);
        pt.setCycleCount(1);

        //animation card1
        RotateTransition rt1 = new RotateTransition(Duration.seconds(2), CARD1);
        rt1.setFromAngle(-35);
        rt1.setToAngle(35);
        rt1.setCycleCount(1);
        rt1.setOnFinished(event -> CARD1.showCard());

        QuadCurve qc1 = new QuadCurve();
        qc1.setStartX(-50f);
        qc1.setStartY(100f);
        qc1.setEndX(150f);
        qc1.setEndY(100f);
        qc1.setControlX(100f);
        qc1.setControlY(50f);

        PathTransition pt1 = new PathTransition();
        pt1.setNode(CARD1);
        pt1.setDuration(Duration.seconds(2));
        pt1.setPath(qc1);
        pt1.setCycleCount(1);

        //animation card2
        RotateTransition rt2 = new RotateTransition(Duration.seconds(2.2), CARD2);
        rt2.setFromAngle(-35);
        rt2.setToAngle(15);
        rt2.setCycleCount(1);
        rt2.setOnFinished(event -> CARD2.showCard());

        QuadCurve qc2 = new QuadCurve();
        qc2.setStartX(-50f);
        qc2.setStartY(100f);
        qc2.setEndX(130f);
        qc2.setEndY(85f);
        qc2.setControlX(100f);
        qc2.setControlY(50f);

        PathTransition pt2 = new PathTransition();
        pt2.setNode(CARD2);
        pt2.setDuration(Duration.seconds(2.2));
        pt2.setPath(qc2);
        pt2.setCycleCount(1);

        //animation card3
        RotateTransition rt3 = new RotateTransition(Duration.seconds(2.4), CARD3);
        rt3.setFromAngle(-35);
        rt3.setToAngle(-5);
        rt3.setCycleCount(1);
        rt3.setOnFinished(event -> CARD3.showCard());

        QuadCurve qc3 = new QuadCurve();
        qc3.setStartX(-50f);
        qc3.setStartY(100f);
        qc3.setEndX(110f);
        qc3.setEndY(80f);
        qc3.setControlX(100f);
        qc3.setControlY(50f);

        PathTransition pt3 = new PathTransition();
        pt3.setNode(CARD3);
        pt3.setDuration(Duration.seconds(2.4));
        pt3.setPath(qc3);
        pt3.setCycleCount(1);

        //animation card4
        RotateTransition rt4 = new RotateTransition(Duration.seconds(2.6), CARD4);
        rt4.setFromAngle(-35);
        rt4.setToAngle(-20);
        rt4.setCycleCount(1);
        rt4.setOnFinished(event -> CARD4.showCard());

        QuadCurve qc4 = new QuadCurve();
        qc4.setStartX(-50f);
        qc4.setStartY(100f);
        qc4.setEndX(90f);
        qc4.setEndY(85f);
        qc4.setControlX(100f);
        qc4.setControlY(50f);

        PathTransition pt4 = new PathTransition();
        pt4.setNode(CARD4);
        pt4.setDuration(Duration.seconds(2.6));
        pt4.setPath(qc4);
        pt4.setCycleCount(1);

        ParallelTransition animation = new ParallelTransition(pt, rt, pt1, rt1, pt2, rt2, pt3, rt3, pt4, rt4);
        animation.play();

        StackPane sp = new StackPane(CARD, CARD4, CARD3, CARD2, CARD1);

        //displaying all the parts on the landing page
        VBox startBox = new VBox(TITLE_LABEL, USERNAME_TEXTFIELD, START_BUTTON, sp);
        startBox.setAlignment(Pos.CENTER);
        startBox.setSpacing(20);
        this.setCenter(startBox);
    }

    /**
     * Starts the animations and fading to the actual game.
     * @param event when the button has been pressed.
     */
    public void start(MouseEvent event) {
        if(USERNAME_TEXTFIELD.getText().isEmpty()) {
            USERNAME_TEXTFIELD.setPromptText("Please enter name.");
            LOG.log(null,"User tried starting with empty username.");
        } else {
            if(USERNAME_TEXTFIELD.getText().matches("^[^\\W_]{3,10}$")) {
                Player player = new Player(USERNAME_TEXTFIELD.getText(), Config.START_SUM);
                LOG.log(player, USERNAME_TEXTFIELD.getText() + " user was created.");
                //saves the name of the user as a new Player
                this.CONTROLLER.setPlayer(player);

                //button color changes on click
                PauseTransition buttonTransition = new PauseTransition(Duration.seconds(0.25));
                buttonTransition.setOnFinished(event2 -> START_BUTTON.setBackground(
                        new Background(
                                new BackgroundImage(
                                        new Image(String.valueOf(this.getClass().getResource("/Images/btn_start.png"))),
                                        BackgroundRepeat.NO_REPEAT,
                                        BackgroundRepeat.NO_REPEAT,
                                        BackgroundPosition.CENTER,
                                        new BackgroundSize(1.0, 1.0, true, true, false, false)
                                )
                        ))
                );
                START_BUTTON.getStyleClass().removeAll("Background");
                START_BUTTON.setBackground(
                        new Background(
                                new BackgroundImage(
                                        new Image(String.valueOf(this.getClass().getResource("/Images/btn_start_onClick.png"))),
                                        BackgroundRepeat.NO_REPEAT,
                                        BackgroundRepeat.NO_REPEAT,
                                        BackgroundPosition.CENTER,
                                        new BackgroundSize(1.0, 1.0, true, true, false, false)
                                )
                        )
                );

                ParallelTransition txtAndBtnFade = new ParallelTransition(
                        getFadeTransition(USERNAME_TEXTFIELD),
                        getFadeTransition(START_BUTTON),
                        getFadeTransition(CARD),
                        getFadeTransition(CARD1),
                        getFadeTransition(CARD2),
                        getFadeTransition(CARD3),
                        getFadeTransition(CARD4)
                );

                //title scaling transition
                ScaleTransition titleScale = new ScaleTransition(Duration.seconds(2), TITLE_LABEL);
                titleScale.setFromX(1);
                titleScale.setFromY(1);
                titleScale.setToX(4);
                titleScale.setToY(4);

                //title fading transition
                FadeTransition titleFade = new FadeTransition(Duration.seconds(1.5), TITLE_LABEL);
                titleFade.setFromValue(10);
                titleFade.setToValue(0);
                titleFade.setDelay(Duration.seconds(0.5));

                //title positioning transition
                TranslateTransition titlePath = new TranslateTransition(Duration.seconds(2.5), TITLE_LABEL);
                titlePath.setFromY(0);
                titlePath.setToY(100);

                ParallelTransition titleFadesAndScales = new ParallelTransition(titlePath, titleFade, titleScale);

                //playing all the transitions
                SequentialTransition seqT = new SequentialTransition(buttonTransition, txtAndBtnFade, titleFadesAndScales);
                seqT.play();

                seqT.setOnFinished(t -> {
                    Main main = new Main(CONTROLLER);
                    this.CONTROLLER.mainPage = main;
                    this.CONTROLLER.changeScene(new Scene(main,Config.SCREEN_WIDTH,Config.SCREEN_HEIGHT));
                });
            } else {
                LOG.log(null, "User did not match the username regex.");
                CustomDialog dialog = new CustomDialog(
                        "Insert valid username",
                        "Please enter a valid username that complies with the following rules:\n" +
                        " - Can consist of lower and capitals\n" +
                        " - Can consist numbers\n" +
                        " - Cannot consist of underscores, hyphens abd spaces\n" +
                        " - Has to be shorter than 10 and larger than 3\n",
                        "Ok",
                        "Close"
                );
                dialog.display();
                dialog.setConfirmEvent(ev -> {
                   dialog.hideStage();
                   USERNAME_TEXTFIELD.setText("");
                });
                dialog.setDeclineEvent(ev -> {
                    dialog.hideStage();
                    USERNAME_TEXTFIELD.setText("");
                });
            }
        }
    }

    /**
     * This method returns a FadeTransition.
     * @param parent on which the transition should be applied.
     * @return the FadeTransition.
     */
    private FadeTransition getFadeTransition(Parent parent) {
        FadeTransition txtFade = new FadeTransition(Duration.seconds(0.2), parent);
        txtFade.setFromValue(10);
        txtFade.setToValue(0);
        return txtFade;
    }
}