package twentyonecoders.GUI;

import javafx.animation.PathTransition;
import javafx.animation.PauseTransition;
import javafx.animation.RotateTransition;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.image.Image;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.scene.paint.ImagePattern;
import javafx.scene.shape.QuadCurve;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Font;
import javafx.util.Duration;

import twentyonecoders.Controller;
import twentyonecoders.GUI.components.CardUI;
import twentyonecoders.GUI.components.Chip;
import twentyonecoders.GUI.components.CustomDialog;
import twentyonecoders.logic.Config;
import twentyonecoders.logic.Hand;
import twentyonecoders.logic.Player;

import java.util.ArrayList;

/**
 * @author Marco Sciacovelli
 */
public class Main extends Pane {

    private final Controller CONTROLLER;
    private final Player PLAYER;
    private final Player DELAER;
    public Label playerName;
    public Label dealerName;
    public Label playerMoney;
    private CardUI firstDealerCard;

    /**
     * Creates the Main UI of the game.
     * @param controller that controls the flow of the game.
     */
    public Main(Controller controller) {
        this.CONTROLLER = controller;
        this.PLAYER = controller.getPlayer();
        this.DELAER = controller.getDealer();

        DELAER.setHand(new Hand(new ArrayList<>()));
        PLAYER.setHand(new Hand(new ArrayList<>()));
        PLAYER.resetBet();

        this.setHeight(1280);
        this.setWidth(720);
        this.setBackground(
                new Background(
                        new BackgroundImage(
                                new Image(String.valueOf(this.getClass().getResource("/Images/menu_bg.png"))),
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                )
        );

        Rectangle deck = new Rectangle();
        deck.setHeight(120);
        deck.setWidth(80);
        deck.setRotate(-35);
        deck.setArcHeight(10);
        deck.setArcWidth(10);
        deck.setStyle("-fx-effect: dropshadow(three-pass-box, rgba(0,0,0,0.25), 10, 0, 4, 4)");
        deck.setFill(
                new ImagePattern(
                        new Image(String.valueOf(this.getClass().getResource("/Images/card_back.png")))
                )
        );

        Chip fivehundredChip = new Chip(500, PLAYER, controller);
        Chip twohundredChip = new Chip(200, PLAYER, controller);
        Chip onehundredChip = new Chip(100, PLAYER, controller);
        Chip fiftyChip = new Chip(50, PLAYER, controller);
        Chip twentyfiveChip = new Chip(25, PLAYER, controller);
        Chip tenChip = new Chip(10, PLAYER, controller);
        Chip fiveChip = new Chip(5, PLAYER, controller);
        Chip twoChip = new Chip(2, PLAYER, controller);
        Chip oneChip = new Chip(1, PLAYER, controller);

        Button allInButton = new Button();
        allInButton.setMinHeight(120);
        allInButton.setMinWidth(80);
        allInButton.setRotate(35);
        allInButton.setBackground(
                new Background(
                    new BackgroundImage(
                            new Image(String.valueOf(this.getClass().getResource("/Images/AllIn.png"))),
                            BackgroundRepeat.NO_REPEAT,
                            BackgroundRepeat.NO_REPEAT,
                            BackgroundPosition.CENTER,
                            new BackgroundSize(1.0, 1.0, true, true, false, false)
                    )
                )
        );
        allInButton.setStyle("-fx-effect: dropshadow(three-pass-box, rgba(0,0,0,0.25), 10, 0, 4, 4)");
        allInButton.setOnMouseClicked(event ->{
            controller.bet(PLAYER, PLAYER.getMoney() - PLAYER.getCurrentBet());
            controller.playSound(String.valueOf(this.getClass().getResource("/Sounds/chips.wav")));
        });

        HBox chipsBox= new HBox(
                deck,
                fivehundredChip,
                twohundredChip,
                onehundredChip,
                fiftyChip,
                twentyfiveChip,
                tenChip,
                fiveChip,
                twoChip,
                oneChip,
                allInButton
        );

        HBox.setMargin(deck, new Insets(20,0,0,0));
        HBox.setMargin(allInButton, new Insets(20,0,0,0));
        chipsBox.setMinWidth(Config.SCREEN_WIDTH);
        chipsBox.setAlignment(Pos.CENTER);
        chipsBox.setSpacing(20);
        this.getChildren().add(chipsBox);

        Button infoButton = new Button();
        infoButton.setMinSize(60, 60);
        infoButton.setMaxSize(60, 60);
        infoButton.setLayoutX(10);
        infoButton.setLayoutY(10);
        infoButton.setBackground(
                new Background(
                        new BackgroundImage(
                                new Image(String.valueOf(this.getClass().getResource("/Images/btn_info.png"))),
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                )
        );
        infoButton.setAlignment(Pos.TOP_RIGHT);
        infoButton.setOnMouseClicked(event -> {
            PauseTransition infoButtonTransition = new PauseTransition(Duration.seconds(0.25));
            infoButtonTransition.setOnFinished(event2 -> infoButton.setBackground(
                    new Background(
                            new BackgroundImage(
                                    new Image(String.valueOf(this.getClass().getResource("/Images/btn_info.png"))),
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundPosition.CENTER,
                                    new BackgroundSize(1.0, 1.0, true, true, false, false)
                            )
                    )
            ));

            infoButton.getStyleClass().removeAll("Background");
            infoButton.setBackground(
                    new Background(
                        new BackgroundImage(
                            new Image(String.valueOf(this.getClass().getResource("/Images/btn_info_onClick.png"))),
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                    )
            );
            infoButtonTransition.play();

            CustomDialog dialog = new CustomDialog(
                    "Game Rules","The goal of blackjack is to beat the dealer's hand without going\n" +
                    "over 21.\n" +
                    "Face cards are worth 10. Aces are worth 1 or 11, whichever\n" +
                    "makes a better hand.\n" +
                    "To 'Hit' is to ask for another card. To 'Stand' is to hold your total\n" +
                    "and end your turn.\n" +
                    "Dealer will hit until his/her cards total 17 or higher.\n" +
                    "If you go over 21 you bust, and the dealer wins regardless of\n" +
                    "the dealer's hand.\n" +
                    "If you are dealt 21 from the start (Ace & 10), you got a blackjack.",
                    "Ok",
                    "Cancel"
            );
            dialog.display();
            dialog.setConfirmEvent(eventE -> dialog.hideStage());
            dialog.setDeclineEvent(eventE -> dialog.hideStage());
        });
        this.getChildren().add(infoButton);

        Button hitButton = new Button();
        hitButton.setMinWidth(110);
        hitButton.setMinHeight(30);
        hitButton.setDisable(true);
        hitButton.setBackground(
                new Background(
                        new BackgroundImage(
                                new Image(String.valueOf(this.getClass().getResource("/Images/btn_hit.png"))),
                                BackgroundRepeat.NO_REPEAT, BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                )
        );

        hitButton.setOnMouseClicked(e -> {
            PauseTransition hitButtonTransition = new PauseTransition(Duration.seconds(0.25));
            hitButtonTransition.setOnFinished(event2 -> hitButton.setBackground(
                    new Background(
                            new BackgroundImage(
                                    new Image(String.valueOf(this.getClass().getResource("/Images/btn_hit.png"))),
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundPosition.CENTER,
                                    new BackgroundSize(1.0, 1.0, true, true, false, false)
                            )
                    )
            ));
            hitButton.getStyleClass().removeAll("Background");
            hitButton.setBackground(
                    new Background(
                            new BackgroundImage(
                                    new Image(String.valueOf(this.getClass().getResource("/Images/btn_hit_onClick.png"))),
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundPosition.CENTER,
                                    new BackgroundSize(1.0, 1.0, true, true, false, false)
                            )
                    )
            );
            hitButtonTransition.play();
            drawCard();
        });

        Button standButton = new Button();
        standButton.setMinWidth(110);
        standButton.setMinHeight(30);
        standButton.setDisable(true);
        standButton.setBackground(
                new Background(
                        new BackgroundImage(
                                new Image(String.valueOf(this.getClass().getResource("/Images/btn_stand.png"))),
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                )
        );

        standButton.setOnMouseClicked(e -> {
            PauseTransition standButtonTransition = new PauseTransition(Duration.seconds(0.25));
            standButtonTransition.setOnFinished(event2 -> standButton.setBackground(
                    new Background(
                            new BackgroundImage(
                                    new Image(String.valueOf(this.getClass().getResource("/Images/btn_stand.png"))),
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundPosition.CENTER,
                                    new BackgroundSize(1.0, 1.0, true, true, false, false)
                            )
                    )
            ));
            standButton.getStyleClass().removeAll("Background");
            standButton.setBackground(
                    new Background(
                            new BackgroundImage(
                                    new Image(String.valueOf(this.getClass().getResource("/Images/btn_stand_onClick.png"))),
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundPosition.CENTER,
                                    new BackgroundSize(1.0, 1.0, true, true, false, false)
                            )
                    )
            );
            standButtonTransition.play();
            playDealer();
        });

        Button readyButton = new Button();
        readyButton.setMinWidth(110);
        readyButton.setMinHeight(30);
        readyButton.setBackground(
                new Background(
                        new BackgroundImage(
                                new Image(String.valueOf(this.getClass().getResource("/Images/btn_ready.png"))),
                                BackgroundRepeat.NO_REPEAT, BackgroundRepeat.NO_REPEAT,
                                BackgroundPosition.CENTER,
                                new BackgroundSize(1.0, 1.0, true, true, false, false)
                        )
                )
        );

        readyButton.setOnMouseClicked(e -> {
            PauseTransition readyButtonTransition = new PauseTransition(Duration.seconds(0.25));
            readyButtonTransition.setOnFinished(event2 -> readyButton.setBackground(
                    new Background(
                            new BackgroundImage(
                                    new Image(String.valueOf(this.getClass().getResource("/Images/btn_ready.png"))),
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundPosition.CENTER,
                                    new BackgroundSize(1.0, 1.0, true, true, false, false)
                            )
                    )
            ));
            readyButton.getStyleClass().removeAll("Background");
            readyButton.setBackground(
                    new Background(
                            new BackgroundImage(
                                    new Image(String.valueOf(this.getClass().getResource("/Images/btn_ready_onClick.png"))),
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundRepeat.NO_REPEAT,
                                    BackgroundPosition.CENTER,
                                    new BackgroundSize(1.0, 1.0, true, true, false, false)
                            )
                    )
            );
            readyButtonTransition.play();

            if(PLAYER.getCurrentBet() > 0) {
                readyButton.setDisable(true);
                hitButton.setDisable(false);
                standButton.setDisable(false);
                allInButton.setDisable(true);
                oneChip.setDisable(true);
                twoChip.setDisable(true);
                fiveChip.setDisable(true);
                tenChip.setDisable(true);
                twentyfiveChip.setDisable(true);
                fiftyChip.setDisable(true);
                onehundredChip.setDisable(true);
                twohundredChip.setDisable(true);
                fivehundredChip.setDisable(true);
                PLAYER.setMoney(PLAYER.getMoney()- PLAYER.getCurrentBet());
                controller.refreshUI();
                drawCard();
                drawCard();
                drawCardDealer(true);
                drawCardDealer(false);
                controller.checkForNaturalBJ();
            } else {
                CustomDialog dialog = new CustomDialog("Bet needed","You need to bet something","Ok","Exit");
                dialog.display();
                dialog.setConfirmEvent(event -> dialog.hideStage());
                dialog.setDeclineEvent(event -> System.exit(0));
            }
        });

        HBox gameControls = new HBox(hitButton, readyButton, standButton);
        gameControls.setAlignment(Pos.CENTER);
        gameControls.setMinWidth(Config.SCREEN_WIDTH);
        gameControls.setSpacing(20);
        gameControls.setLayoutY(Config.SCREEN_HEIGHT - 75);
        this.getChildren().add(gameControls);

        dealerName = new Label(DELAER.getName());
        dealerName.setMinWidth(Config.SCREEN_WIDTH);
        dealerName.setAlignment(Pos.CENTER);
        dealerName.setLayoutY(Config.DEALER_Y_CENTER + (Config.CARD_HEIGHT / 2.0) + 20);
        dealerName.setTextFill(Color.rgb(206, 255, 26));
        dealerName.setFont(new Font("Arial Bold", 20));
        this.getChildren().add(dealerName);

        playerName = new Label(PLAYER.getName() + ": " + PLAYER.getCurrentBet() + "$");
        playerName.setMinWidth(Config.SCREEN_WIDTH);
        playerName.setAlignment(Pos.CENTER);
        playerName.setLayoutY(Config.PLAYER_Y_CENTER - (Config.CARD_HEIGHT / 2.0) - 40);
        playerName.setTextFill(Color.rgb(206, 255, 26));
        playerName.setFont(new Font("Arial Bold", 20));
        this.getChildren().add(playerName);

        playerMoney = new Label(PLAYER.getMoney() + "$");
        playerMoney.setLayoutX((Config.SCREEN_WIDTH / 2.0) * 1.75);
        playerMoney.setLayoutY(30);
        playerMoney.setAlignment(Pos.CENTER_RIGHT);
        playerMoney.setTextFill(Color.rgb(206, 255, 26));
        playerMoney.setFont(new Font("Arial Bold", 40));
        this.getChildren().add(playerMoney);
    }

    /**
     * Draws a card for the player and adds it to it's hand.
     */
    private void drawCard() {
        CONTROLLER.playSound(String.valueOf(this.getClass().getResource("/Sounds/giving_cards.wav")));
        CardUI card = new CardUI(this.CONTROLLER.getDeck().draw());
        card.setRotate(-35);
        PLAYER.getHand().addCard(card.getCard());
        QuadCurve line = new QuadCurve(
                Config.DECK_X_CENTER,
                Config.DECK_Y_CENTER,
                Config.DECK_X_CENTER,
                Config.PLAYER_Y_CENTER,
                Config.PLAYER_X_CENTER + (40 * this.CONTROLLER.getPlayer().getHand().getCards().size()),
                Config.PLAYER_Y_CENTER);

        PathTransition transition = new PathTransition();
        transition.setNode(card);
        transition.setDuration(Duration.seconds(2));
        transition.setPath(line);
        transition.setOnFinished(e -> {
            CONTROLLER.playSound(String.valueOf(this.getClass().getResource("/Sounds/flip.wav")));
            card.showCard();
            if (CONTROLLER.playerBusted())
                CONTROLLER.checkBusted();
        });
        transition.play();

        RotateTransition rt = new RotateTransition(Duration.millis(2000), card);
        rt.setByAngle(35);

        rt.play();
        this.getChildren().add(card);
    }

    /**
     * Draws a card for the dealer and adds it to it's hand
     * @param facedown if the card should face down.
     */
    private void drawCardDealer(boolean facedown) {
        CONTROLLER.playSound(String.valueOf(this.getClass().getResource("/Sounds/giving_cards.wav")));
        CardUI card = new CardUI(this.CONTROLLER.getDeck().draw());
        card.setRotate(-35);
        DELAER.getHand().addCard(card.getCard());
        QuadCurve line = new QuadCurve(
                Config.DECK_X_CENTER,
                Config.DECK_Y_CENTER,
                Config.DECK_X_CENTER,
                Config.DEALER_Y_CENTER,
                Config.DEALER_X_CENTER + (40 * this.CONTROLLER.getDealer().getHand().getCards().size()),
                Config.DEALER_Y_CENTER);

        PathTransition transition = new PathTransition();
        transition.setNode(card);
        transition.setDuration(Duration.seconds(2));
        transition.setPath(line);
        transition.setOnFinished(e -> {
            if(!facedown) {
                CONTROLLER.playSound(String.valueOf(this.getClass().getResource("/Sounds/flip.wav")));
                card.showCard();
            } else {
                firstDealerCard = card;
            }
        });
        transition.play();

        RotateTransition rt = new RotateTransition(Duration.millis(2000), card);
        rt.setByAngle(35);

        rt.play();
        this.getChildren().add(card);
    }

    /**
     * Plays the dealer and draws a card if his hand value, is lower than 17.
     */
    public void playDealer() {
        if(DELAER.getHand().getValue() < 17) {
            drawCardDealer(false);
            playDealer();
        } else {
            firstDealerCard.showCard();
            CONTROLLER.checkHand();
        }
    }
}
