package twentyonecoders.GUI.components;

import javafx.scene.Parent;
import javafx.scene.image.Image;
import javafx.scene.paint.ImagePattern;
import javafx.scene.shape.Rectangle;

import twentyonecoders.logic.Card;
import twentyonecoders.logic.Config;

/**
 * @author Marco Sciacovelli
 */
public class CardUI extends Parent {

    private final Card CARD;
    private final ImagePattern FRONT_PATTERN;
    private final Rectangle CARD_RECT;

    /**
     * Creates a UI card according to the passed card logic.
     * @param card values that should be displayed.
     */
    public CardUI(Card card) {
        this.CARD = card;
        CARD_RECT = new Rectangle(Config.CARD_WIDTH, Config.CARD_HEIGHT);
        CARD_RECT.setArcWidth(Config.CARD_RADIUS);
        CARD_RECT.setArcHeight(Config.CARD_RADIUS);
        ImagePattern backPattern = new ImagePattern(
                new Image(String.valueOf(this.getClass().getResource("/Images/card_back.png")))
        );
        FRONT_PATTERN = new ImagePattern(
                new Image(String.valueOf(this.getClass().getResource(
                        "/Images/" +
                        card.getRank().toString().toLowerCase() +
                        "_" +
                        card.getSuit().toString().toLowerCase() +
                        ".png"))
                )
        );
        CARD_RECT.setFill(backPattern);
        CARD_RECT.setStyle("-fx-effect: dropshadow(three-pass-box, rgba(0,0,0,0.8), 10, 0, 2, 2)");
        this.getChildren().add(CARD_RECT);
    }

    /**
     * Returns the card that is linked to the UI.
     * @return the card logic.
     */
    public Card getCard() {
        return CARD;
    }

    /**
     * Show the card and refresh the card, showing the front side.
     */
    public void showCard() {
        CARD_RECT.setFill(FRONT_PATTERN);
    }
}
