package twentyonecoders.GUI.components;

import javafx.event.EventHandler;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.image.Image;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Line;
import javafx.scene.text.Font;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;

import twentyonecoders.logic.Config;

/**
 * @author Marco Sciacovelli
 */
public class CustomDialog extends Stage {

    private final String HEADER;
    private final String TEXT;
    private final String CONFIRM_TEXT;
    private final String DECLINE_TEXT;
    private Button confirmButton;
    private Button declineButton;

    /**
     * It creates a Dialog with the passed parameters.
     * @param header text that will be displayed.
     * @param text content that will be displayed.
     * @param confirmText that will be displayed in the confirmation Button.
     * @param declineText that will be displayed int the declination Button.
     */
    public CustomDialog(String header, String text, String confirmText, String declineText) {
        this.HEADER = header;
        this.TEXT = text;
        this.CONFIRM_TEXT = confirmText;
        this.DECLINE_TEXT = declineText;
    }

    /**
     * Sets the styling of the dialog and shows it.
     */
    public void display() {
        this.initModality(Modality.APPLICATION_MODAL);

        Pane pane = new Pane();
        pane.setStyle("-fx-background-radius: 30px; -fx-background-color: white;");
        pane.setMinWidth(Config.SCREEN_WIDTH / 3.0);
        pane.setMinHeight(Config.SCREEN_HEIGHT / 3.0);

        Label headerLabel = new Label(HEADER);
        headerLabel.setLayoutX(30);
        headerLabel.setLayoutY(30);
        headerLabel.setFont(new Font("Arial Bold", 30));
        pane.getChildren().add(headerLabel);

        Line divider = new Line();
        divider.setStartX(20);
        divider.setStartY(70);
        divider.setEndX((Config.SCREEN_WIDTH / 3.0) - 20);
        divider.setEndY(70);
        pane.getChildren().add(divider);

        Label textLabel = new Label(TEXT);
        textLabel.setLayoutX(30);
        textLabel.setLayoutY(80);
        textLabel.setMinWidth((Config.SCREEN_WIDTH / 3.0) - 60);
        textLabel.setFont(new Font("Arial", 15));
        pane.getChildren().add(textLabel);

        confirmButton = new Button(CONFIRM_TEXT);
        confirmButton.setMinWidth(100);
        confirmButton.setLayoutX((Config.SCREEN_WIDTH / 14.0));
        confirmButton.setLayoutY((Config.SCREEN_HEIGHT / 3.0) - 30);
        confirmButton.setStyle("-fx-background-color: lightgray");
        pane.getChildren().add(confirmButton);

        declineButton = new Button(DECLINE_TEXT);
        declineButton.setMinWidth(100);
        declineButton.setLayoutX((Config.SCREEN_WIDTH / 4.0) - 50);
        declineButton.setLayoutY((Config.SCREEN_HEIGHT / 3.0) - 30);
        declineButton.setStyle("-fx-background-color: lightgray");
        pane.getChildren().add(declineButton);

        Scene scene = new Scene(pane, Config.SCREEN_WIDTH / 3.0, Config.SCREEN_HEIGHT / 3.0);
        scene.setFill(Color.TRANSPARENT);

        this.setResizable(false);
        this.initStyle(StageStyle.TRANSPARENT);
        this.setScene(scene);
        this.getIcons().add(new Image(String.valueOf(this.getClass().getResource("/Images/icon.png"))));
        this.show();
    }

    /**
     * Sets the Action, that should be fired when the confirmation button is pressed.
     * @param confirmEvent that should be executed.
     */
    public void setConfirmEvent(EventHandler<? super MouseEvent> confirmEvent) {
        confirmButton.setOnMouseClicked(confirmEvent);
    }

    /**
     * Sets the Action, that should be fired when the decline button is pressed.
     * @param declineEvent that should be executed.
     */
    public void setDeclineEvent(EventHandler<? super MouseEvent> declineEvent) {
        declineButton.setOnMouseClicked(declineEvent);
    }

    /**
     * Hides and closes the dialog.
     */
    public void hideStage() {
        this.hide();
        this.close();
    }
}
