package twentyonecoders.logic;

import javafx.scene.Parent;

/**
 * @author Marco Sciacovelli
 */
public class Card extends Parent {

    /**
     * The rank of the card. goes from 2 to 11.
     * If it is an Ace, we must check if 1 or 11
     */
    public enum Rank {
        TWO(2),
        THREE(3),
        FOUR(4),
        FIVE(5),
        SIX(6),
        SEVEN(7),
        EIGHT(8),
        NINE(9),
        TEN(10),
        JACK(10),
        QUEEN(10),
        KING(10),
        ACE(11);

        final int value;

        Rank(int value) {
            this.value = value;
        }
    }

    /**
     * The Possible Suits:
     *  - Hearts
     *  - Diamonds
     *  - Clubs
     *  - Spades
     */
    public enum Suit {
        HEARTS,
        DIAMONDS,
        CLUBS,
        SPADES
    }


    //Constants regarding the value of the card.
    private final Suit SUIT;
    private final Rank RANK;

    /**
     * Card has a suit and a rank, can be shown or hidden.
     * @param suit of the card
     * @param rank of the card
     */
    public Card(Suit suit, Rank rank) {
        SUIT = suit;
        RANK = rank;
    }

    /**
     * Returns the rank of the card.
     * @return rank
     */
    public Rank getRank() {
        return RANK;
    }

    /**
     * Returns the suit of the card.
     * @return suit
     */
    public Suit getSuit() {
        return this.SUIT;
    }

    @Override
    public String toString() {
        return "Card{" +
                "SUIT=" + SUIT +
                ", RANK=" + RANK +
                '}';
    }
}
