package twentyonecoders.logic;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Random;

/**
 * @author Leo Kerschbaumer
 */
public class Deck {

    /**
     * The remaining cardStates in the deck that can be drawn.
     */
    private final ArrayList<Card> REMAINING = new ArrayList<>();

    /**
     * The cardStates that have already been drawn,
     */
    private final ArrayList<Card> DRAWN = new ArrayList<>();

    /**
     * Creates a new Deck and initiates all the cardStates of a deck.
     */
    public Deck() {
        Arrays.stream(Card.Suit.values()).forEach(suit ->
                Arrays.stream(Card.Rank.values()).forEach(rank ->
                        this.REMAINING.add(new Card(suit, rank))
                )
        );
    }

    /**
     * Shuffles the remaing cardStates in the deck.
     */
    public void shuffle() {
        Collections.shuffle(this.REMAINING);
    }

    /**
     * Draws a card from the remaining card pool.
     * @return -the drawn card
     *         -null if no cardStates are left
     */
    public Card draw() {
        Card ret = null;

        if(this.REMAINING.size() > 0) {
            Random rand = new Random();
            ret = this.REMAINING.get(rand.nextInt(this.REMAINING.size()));
            this.DRAWN.add(ret);
        }

        return ret;
    }

    /**
     * Returns a formatted String representation of the Deck:
     *      Deck {remaining=..., drawn=...}
     * @return formatted String
     */
    @Override
    public String toString() {
        return "Deck{" +
                "remaining=" + REMAINING +
                ", drawn=" + DRAWN +
                '}';
    }
}