package twentyonecoders.logic;

import java.util.ArrayList;

/**
 * @author Leo Kerschbaumer
 */
public class Hand {

    /**
     * The cardStates in hand.
     */
    private final ArrayList<Card> CARDS;

    /**
     * Keep track of aces because they can change value later on as well.
     */
    private int aces;

    /**
     * The value of the cardStates in the hand.
     */
    private int value;

    /**
     * Creates a Hand with the passed cardState.
     * @param cardStates start cardStates for the hand.
     */
    public Hand(ArrayList<Card> cardStates) {
        this.CARDS = cardStates;
        this.value = 0;
        this.aces = 0;
    }

    /**
     * Adds a card to the hand and adds the value.
     * If a player would be over 21 but has an ace 10 will be subtracted to make the ace count as 1.
     * @param card the card to add.
     */
    public void addCard(Card card) {
        if(card != null) {
            this.CARDS.add(card);

            if(card.getRank() == Card.Rank.ACE) {
                this.aces++;
            }

            this.value += card.getRank().value;

            //if the player would be over 21 but has an ace the value is decreased by 10
            if(this.value > 21 && this.aces > 0) {
               this.aces--;
               this.value -= 10;
            }
        }
    }

    /**
     * Resets the hands cardStates and value.
     */
    public void resetHand() {
        this.CARDS.clear();
        this.value = 0;
        this.aces = 0;
    }

    /**
     * This method returns an ArrayList with all the cards contained in the hand.
     * @return the cards in the hand.
     */
    public ArrayList<Card> getCards() {
        return CARDS;
    }

    /**
     * This method returns the value of your hand.
     * @return the value of your hand.
     */
    public int getValue() {
        return value;
    }
}
