package twentyonecoders.logic;

/**
 * @author Lisa Mussner
 */
public class Player {
    private final String NAME;
    private Hand hand;
    private int money, currentBet;

    /**
     * Creates a player with the passed parameter.
     * @param name of the player.
     * @param startSum of the player.
     */
    public Player(String name, int startSum) {
        this.NAME = name;
        this.setMoney(startSum);
    }

    /**
     * This method returns the name of the player.
     * @return the name of the player.
     */
    public String getName() {
        return NAME;
    }

    /**
     * This method returns the balance of the player.
     * @return the balance of the player.
     */
    public int getMoney() {
        return money;
    }

    /**
     * This method sets the balance of the player to the passed value.
     * @param money the balance to be set.
     */
    public void setMoney(int money) {
        this.money = money;
    }

    /**
     * This method returns the hand of the player.
     * @return the hand of the player.
     */
    public Hand getHand() {
        return hand;
    }

    /**
     * Sets the player's Hand to the passed value.
     * @param hand to be set.
     */
    public void setHand(Hand hand) {
        this.hand = hand;
    }

    /**
     * This method returns the current bet that the player has set.
     * @return the player's current bet.
     */
    public int getCurrentBet() {
        return currentBet;
    }

    /**
     * This method sets the player's bet to the passed value.
     * @param currentBet to be set.
     */
    public void setCurrentBet(int currentBet) {
        this.currentBet += currentBet;
    }

    /**
     * Resets the current bet to 0.
     */
    public void resetBet() {
        this.currentBet = 0;
    }
}
